#!/bin/sh
# Copyright (c) 2002-2011 by Rack-Soft, LLC
# Copyright (c) 2002-2011 by 4PSA (www.4psa.com)
# All rights reserved


# Modified: $DateTime: 2013/07/01 16:49:25 $
# Revision: $Revision: #12 $ with $Change: 98215 $

# DUMPS Plesk DNS Zones in the 4PSA DNS Manager 3 dump format
# Compatible with Plesk 7.5, 8.x installations
# Compatible with DNS Manager 3.0.0 and above
# Description: for every zone in the Plesk database, the scripts dumps DNS records

# --------------------------------------------------------------------------------
#
# Please modify the default DNS records target (backup file)
# This will be remote update location in 4PSA DNS Manager 
# You must include the full path!
# The program can also take a dump destination as the argument

# NOTE that if 1 argument is given dump_file will be the argument
dump_file="dump_full_recs.txt"
#temporary files
dump_temp_file="$dump_file-$$"
dump_ptr_file="$dump_file-$$.ptr"
#

# Dump type can be <masters/slaves/both>
dump="both"

# Converts master zones to slaves in dump (for DNS Manager 3 acting as slave for a bind server)
# Works only when
# dump=masters/both
# dump_allow_transfers=yes/no
# dump_masters=yes/no
masters2slaves="no"

# Dump 'master' records from existing zones and includes them in the dump
# Works only when:
# dump=slaves/both
# masters2slaves=yes/no (Zones transformed from master2slave can not have a
# masters record because it does not exist in database!)
# dump_allow_transfers=yes/no
dump_masters="yes"


# Dump SOA records from Plesk database 
dump_soa="no"

# Dump reverse zones
dump_reverse="yes"

# Ignore the zone status in the Plesk database (zone and/or domain can be disabled in Plesk)
ignore_dns_zones_status="no"

# NS list for reverse zones
#REVERSE_NS="ns1.example.com ns2.example.com"
REVERSE_NS=""

# DO NOT MODIFY ANYTHING BELOW THIS LINE UNLESS YOU KNOW WHAT YOU ARE DOING
# -------------------------------------------------------------------------

get_config()
{
	#check local vars
	[ -n "$dump" ]||dump="masters"
	[ -n "$masters2slaves" ]||masters2slaves="yes"
	[ -n "$dump_masters" ]||dump_masters="no"
	[ -n "$dump_allow_transfers" ]||dump_allow_transfers="no"
	[ -n "$dump_soa" ]||dump_soa="no"
	[ -n "$dump_reverse" ]||dump_reverse="yes"
	[ -n "$ignore_dns_zones_status" ]||ignore_dns_zones_status="no"

	if [ ! -f "${config}" ];then
		echo 'Config file does not exist.'
		exit 1
	fi    
	    
	mysql_dir="`awk '/MYSQL_BIN_D/{print $2}' ${config}`"
	if [ -z "${mysql_dir}" ];then
		echo 'Unable to find Plesk mysql dir.'
		exit 1
	fi
    
	named_dir="`awk '/NAMED_RUN_ROOT_D/{print $2}' ${config}`"
	if [ -z "${named_dir}" ];then
		echo 'Unable to find Named zones dir.'
	fi

	psa_root_d="`awk '/PRODUCT_ROOT_D/{print $2}' ${config}`"
	if [ -f "${psa_root_d}/version" ];then
		OLDVER="`awk -F"." '{if ($1 > 7){print "0" }else{ print "1"}}' ${psa_root_d}/version`"
		if [ "`awk -F"." '{print $1$2}' ${psa_root_d}/version`" -ge "115" ];then 
			domainaliases="domain_aliases"
		else
			domainaliases="domainaliases"
		fi
	else
		echo "Could not determine plesk version"
		exit 1
	fi
	zone_dir="${named_dir}/var"
	if [ ! -d "${zone_dir}" ];then
		echo "Total domains zones will not be exported"
	fi
}

is_root()
{
	current_user="`id`"
	case ${current_user} in
		uid=0\(root\)*)
				;;
			    *)
			    	echo "$0: You must have root privileges to run this script"
				echo "Su as root and try again"
				echo
				exit 1
				;;
	esac
}

get_masters()
{
	# get domains id
	if [ "$ignore_dns_zones_status" = "yes" ];then
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from domains WHERE dns_type="master";' psa  |awk '{print $1}'`"
		else
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from dns_zone WHERE type="master" and id!=1;' psa  |awk '{print $1}'`"
		fi
	else
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from domains WHERE dns_type="master" AND dns_zone="true";' psa  |awk '{print $1}'`"
		else
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"SELECT dns_zone.id from dns_zone,domains WHERE (dns_zone.id=domains.dns_zone_id AND dns_zone.type='master' AND domains.status=0 AND dns_zone.status=0)\
					UNION SELECT dns_zone.id FROM dns_zone,$domainaliases WHERE (dns_zone.id=$domainaliases.dns_zone_id AND dns_zone.type='master' AND $domainaliases.status=0 AND dns_zone.status=0);" psa |awk '{print $1}'`"
		fi
	fi

	local i=""
	if [ "$masters2slaves" = "yes" ];then
	    local mastertype="slave"
	else
	    local mastertype="master"
	fi	
    
	for i in $domain_ids; do
		local domain_name=''	
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_name="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select name from domains where id=$i;" psa | awk '{printf("%s", $1)}'`"
		else   
			local domain_name="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select name from dns_zone where id=$i;" psa | awk '{printf("%s", $1)}'`"
		fi

		if [ ! -z $domain_name ];then
			echo "$domain_name.|$mastertype {" >>$dump_temp_file
			if [ "$OLDVER" -eq "1" ];then
				if [ "$dump_reverse" = "yes" ];then
				    $mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host,val,opt from dns_recs where dom_id=$i and type='PTR' order by id;" psa|awk '{
					printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
				    }' >>$dump_ptr_file
				fi
			else
				if [ "$dump_reverse" = "yes" ];then
					$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host,val,opt from dns_recs where dns_zone_id=$i and type='PTR' and dns_zone_id not in (select dns_zone_id from $domainaliases where dns_zone_id=$i) order by id;" psa | awk '{
						 printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
					}' >> $dump_ptr_file
				fi
			fi
			if	[ "$masters2slaves" = "no" ];then
				if [ "$OLDVER" -eq "1" ]; then 
					$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host,val,opt from dns_recs where dom_id=$i order by id;" psa | awk '{
					if (toupper($1)!="TXT") {
						if(ptr=="yes") printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
						else if (toupper($1)!="PTR") printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
					} else { split ($0,a,"\""); printf ("|%s| |%s| |%s| ||\n",$1,$2,a[2]); }
					}' ptr=0 >> $dump_temp_file
				else    
					if [ "$dump_soa" = "yes" ];then
					
						$mysql_dir/mysql -uadmin -p$admin_passwd  -Nse "select refresh,retry,expire,minimum,ttl,serial  from dns_zone where id=$i" psa|while read  soa;do
							soa_refresh="`echo $soa|awk '{print $1}'`"
							soa_retry="`echo $soa|awk '{print $2}'`"
							soa_expire="`echo $soa|awk '{print $3}'`"
							soa_min_ttl="`echo $soa|awk '{print $4}'`"
							soa_def_ttl="`echo $soa|awk '{print $5}'`"
							soa_serial="`echo $soa|awk '{print $6}'`"
							echo "|REFRESH| |${soa_refresh}| || ||">>$dump_temp_file
							echo "|RETRY| |${soa_retry}| || ||">>$dump_temp_file
							echo "|EXPIRE| |${soa_expire}| || ||">>$dump_temp_file
							echo "|MIN_TTL| |${soa_min_ttl}| || ||">>$dump_temp_file
							echo "|DEFAULT_TTL| |${soa_def_ttl}| || ||">>$dump_temp_file
							echo "|SERIAL| |${soa_serial}| || ||">>$dump_temp_file
						done					
					fi
					$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host,val,opt from dns_recs where dns_zone_id=$i order by id;" psa | awk '{
					if (toupper($1)=="SRV") {
						 printf ("|%s| |%s| |%s| |%s| |%s| |%s| ||\n",$1,$2,$3,$4,$5,$6);
					}
					else if (toupper($1)!="TXT") {
						if(ptr=="yes") printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
							else if (toupper($1)!="PTR") printf ("|%s| |%s| |%s| |%s|\n",$1,$2,$3,$4);
					} else { type=$1;host=$2;str=$0;sub($1,"",str);sub($2,"",str);gsub(/^[ \t]+|[ \t]+$/,"",str);printf ("|%s| |%s| |%s| ||\n",type,host,str); }
					}' ptr=0 >> $dump_temp_file
				fi
			fi
		echo "}" >> $dump_temp_file
	fi
	done
	# dump 4PSA Total Domains parkings
	# verify 4psadomains db exist
	buff="`$mysql_dir/mysql -uadmin -p$admin_passwd -e"show databases;" | grep -i "4psadomains"`"
	if [ -n "$buff" ]; then
		domain_names="`$mysql_dir/mysql -uadmin -p$admin_passwd -e"select name from subdomains where type='1';" 4psadomains | sed -e 1d | awk '{print $1}'`"
		for i in $domain_names; do
			if [ ! -f "$zone_dir/$i" ]; then
				echo "Warning - Config file for domain $i does not exist."
			else        
			echo "$i.|$mastertype {" >>$dump_temp_file
			if	[ "$masters2slaves" = "no" ];then
			
				buff="`awk '
					! /^@/&&/[[:blank:]]IN[[:blank:]]/ {
					if (NF==5) printf("|%s| |%s| |%s| |%s|\n", $3,$1,$5,$4); 
					else printf("|%s| |%s| |%s| ||\n", $3,$1,$4);
					}' $zone_dir/$i`"
			
				echo "$buff" >> $dump_temp_file
			fi
			echo "}" >> $dump_temp_file
		fi	
		done
	fi

}

get_slaves()
{
	# get domains id
	if [ "$ignore_dns_zones_status" = "yes" ];then
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from domains WHERE dns_type="slave";' psa  |awk '{print $1}'`"
		else
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from dns_zone WHERE type="slave";' psa |awk '{print $1}'`"
		fi
	else
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse'select id from domains WHERE dns_type="slave" AND dns_zone="true";' psa  |awk '{print $1}'`"
		else
			local domain_ids="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"SELECT dns_zone.id from dns_zone,domains WHERE (dns_zone.id=domains.dns_zone_id AND dns_zone.type='slave' AND domains.status=0 AND dns_zone.status=0)\
					UNION SELECT dns_zone.id FROM dns_zone,$domainaliases WHERE (dns_zone.id=$domainaliases.dns_zone_id AND dns_zone.type='slave' AND $domainaliases.status=0 AND dns_zone.status=0);" psa  |awk '{print $1}'`"
		fi
	fi

	local i=""
	for i in $domain_ids; do
		local domain_name=''	
		if [ "$OLDVER" -eq "1" ] ; then 
			local domain_name="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select name from domains where id=$i;" psa  | awk '{printf("%s", $1)}'`"
		else   
			local domain_name="`$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select name from dns_zone where id=$i;" psa  | awk '{printf("%s", $1)}'`"
		fi

		if [ ! -z $domain_name ];then
				echo "$domain_name.|slave {" >>$dump_temp_file
			if [ "$dump_masters" = "yes" ];then
				if [ "$OLDVER" -eq "1" ]; then 
					$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host from dns_recs where where lower(type)='master' and dom_id=$i order by id;" psa | awk '{
							printf ("|%s| |%s| || ||\n",toupper($1),$2);
					}' >> $dump_temp_file
				else    
					$mysql_dir/mysql -uadmin -p$admin_passwd -Nse"select type,host from dns_recs where lower(type)='master' and dns_zone_id=$i order by id;" psa | awk '{
							printf ("|%s| |%s| || ||\n",toupper($1),$2);
					}' >> $dump_temp_file
				fi
			fi
		echo "}" >> $dump_temp_file
	fi
	done
}

dump_ptr(){
    local ptr_file="$1"
    local dump_file="$2"
    local r_ns;
    awk '/\|PTR\|/{ip=$2;gsub(/\|/,"",ip);split(ip,net,".");printf (net[3]"."net[2]"."net[1]".in-addr.arpa\t"net[1]"."net[2]"."net[3]"\n")};' $dump_ptr_file|sort -u|while read line; do
	local zone="`echo $line|awk '{print $1}'`"
	local net="`echo $line|awk '{print $2}'`"
	if [ "$masters2slaves" = "yes" ];then
	    echo "${zone}.|slave {" >>$dump_file
	else
	    echo "${zone}.|master {" >>$dump_file
	    if [ -n "${REVERSE_NS}" ];then 
		for r_ns in ${REVERSE_NS}; do 
			echo "|NS| |${zone}.| |${r_ns}.| ||" >> $dump_file
		done
	    fi
	    awk "/$net/{inet=\$2;gsub(/\|/,\"\",inet);split(inet,ip,\".\");print \$1,\"|\"ip[4]\".\"ip[3]\".\"ip[2]\".\"ip[1]\".in-addr.arpa.|\",\$3,\$4}" $dump_ptr_file >>$dump_file
	fi
	echo "}" >>$dump_file
    done
    

}

# main

# Plesk configuration files
config="/etc/psa/psa.conf"
admin_passwd="`cat /etc/psa/.psa.shadow`"

is_root
get_config

if [ $# -eq 1 ]; then
	echo "Using $1 as a dump file"
	dump_file=$1
	dump_temp_file="$dump_file-$$"
fi

if [ $# -gt 1 ]; then
	exit 1
fi

echo -n > $dump_temp_file


case "$dump" in 
	masters)
		get_masters
 	;;
 	slaves)
 		get_slaves
 	;;	
	both)
		get_masters
 		get_slaves
	;;	
esac 	

if [ "$dump_reverse" = "yes" ];then
    [ -f "$dump_ptr_file" ] && dump_ptr "$dump_ptr_file" "$dump_temp_file"
fi
rm -f "$dump_ptr_file"     
mv $dump_temp_file $dump_file >>/dev/null 2>&1
index="`grep -c "{" $dump_file`"
echo "$index complete zones dumped succesfully!"

exit 0
